package com.example.furintiture.ui.screen.detail

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.layout.wrapContentSize
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Close
import androidx.compose.material3.Button
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import coil3.compose.AsyncImage
import coil3.request.ImageRequest
import coil3.request.crossfade
import com.example.furintiture.R
import com.example.furintiture.ui.common.FurnitureButton
import com.example.furintiture.ui.screen.Detail
import com.example.furintiture.ui.theme.Neutral_10
import com.example.furintiture.ui.theme.Neutral_20
import com.example.furintiture.ui.theme.Neutral_40
import kotlinx.coroutines.launch
import java.math.BigDecimal

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun DetailScreen(
    detail: Detail,
    onClosePressed: () -> Unit
) {
    val snackbarHostState  = remember { SnackbarHostState() }
    val coroutineScope = rememberCoroutineScope()
    val viewModel = viewModel {
        DetailScreenViewModel(detail.furnitureId, detail.uuid)
    }
    Scaffold(
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState)
        },
        topBar = {
            TopAppBar(
                title = {},
                navigationIcon = {
                    IconButton (
                        onClick = {
                            onClosePressed()
                        }
                    ){
                        Icon(
                            imageVector = Icons.Default.Close,
                            contentDescription = null
                        )
                    }
                }
            )
        },
        bottomBar = {
            Column(
                modifier = Modifier
                    .fillMaxWidth()
                    .background(color = Neutral_20, shape = RoundedCornerShape(topStart = 16.dp, topEnd = 16.dp))
                    .padding(20.dp)
            ) {
                FurnitureButton(
                    modifier = Modifier.height(60.dp),
                    text = "Добавить в корзину",
                ) {
                    viewModel.addToCart()
                }
            }
        },
        contentColor = Neutral_20
    ) { paddingValues ->
        DetailScreenContent(viewModel, paddingValues){
            coroutineScope.launch {
                snackbarHostState.showSnackbar(message = it)
            }
        }
    }
}

@Composable
fun DetailScreenContent(
    detailScreenViewModel: DetailScreenViewModel,
    paddingValues: PaddingValues,
    errorCallback: (String) -> Unit

){
    val state = detailScreenViewModel.detailScreenState.collectAsState()
    LaunchedEffect(state.value.error) {
        state.value.error?.let(errorCallback)
    }
    Column(
        modifier = Modifier
            .padding(paddingValues)
            .fillMaxSize()
            .background(color = Neutral_20)
        ,
        verticalArrangement = Arrangement.Center
    ) {
        Spacer(modifier = Modifier.height(16.dp))
        AsyncImage(
            modifier = Modifier
                .size(350.dp, 350.dp)
                .align(Alignment.CenterHorizontally)
            ,
            model = ImageRequest
                .Builder(LocalContext.current)
                .data(state.value.furnitureResponse.url)
                .crossfade(true)
                .build(),
            placeholder = painterResource(R.drawable.images),
            contentScale = ContentScale.FillWidth,
            contentDescription = null
        )
        Spacer(modifier = Modifier.height(16.dp))
        Column(
            modifier = Modifier
                .fillMaxWidth()
                .background(Color.White, shape = RoundedCornerShape(
                    topStart = 16.dp,
                    topEnd = 16.dp)
                )
                .weight(1f)
                .padding(horizontal = 20.dp, vertical = 10.dp)
        ) {
            Text(
                text = state.value.furnitureResponse.name,
                style = TextStyle(
                    color = com.example.furintiture.ui.theme.Text,
                    fontFamily = FontFamily(Font(R.font.manrope)),
                    fontSize = 28.sp
                )
            )
            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.Start,
                verticalAlignment = Alignment.CenterVertically
            ) {
                Text(
                    text = "${state.value.furnitureResponse.price} ₽",
                    style = TextStyle(
                        color = com.example.furintiture.ui.theme.Text,
                        fontFamily = FontFamily(Font(R.font.manrope)),
                        fontSize = 40.sp,
                        fontWeight = FontWeight.Bold
                    )
                )
                Spacer(modifier = Modifier.width(8.dp))
                Text(
                    text = "${state.value.furnitureResponse.sale.multiply(BigDecimal(100)).toInt()}% OFF",
                    modifier = Modifier
                        .background(
                            color = Color.Red,
                            shape = RoundedCornerShape(10.dp)
                        )
                        .padding(
                            vertical = 4.dp,
                            horizontal = 6.dp
                        ),
                    style = TextStyle(
                        color = Color.White,
                        fontWeight = FontWeight.SemiBold,
                        fontFamily = FontFamily(Font(R.font.manrope)),
                        fontSize = 20.sp
                    )
                )
            }
            Text(
                text = state.value.furnitureResponse.description,
                style = TextStyle(
                    color = com.example.furintiture.ui.theme.Text,
                    fontWeight = FontWeight.Normal,
                    fontFamily = FontFamily(Font(R.font.manrope)),
                    fontSize = 18.sp
                )
            )
        }
    }
}