package com.example.furintiture.ui.screen.login

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.example.furintiture.R
import com.example.furintiture.data.DataStoreSettings
import com.example.furintiture.ui.common.FurnitureButton
import com.example.furintiture.ui.common.FurniturePasswordTextField
import com.example.furintiture.ui.common.FurnitureTextField
import com.example.furintiture.ui.theme.Neutral_20
import com.example.furintiture.ui.theme.Neutral_70
import com.example.furintiture.ui.theme.Primary
import kotlinx.coroutines.launch

@Composable
fun LoginScreen(
    dataStoreSettings: DataStoreSettings,
    onNavigateToRegistration: () -> Unit
) {
    val snackbarHostState  = remember { SnackbarHostState() }
    val coroutineScope = rememberCoroutineScope()
    val viewModel = viewModel { LoginScreenViewModel(dataStoreSettings) }
    Scaffold(
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState)
        },
        contentColor = Neutral_20
    ) { paddingValues ->
        LoginScreenContent(viewModel, paddingValues, onNavigateToRegistration){
            coroutineScope.launch {
                snackbarHostState.showSnackbar(message = it)
            }
        }
    }
}

@Composable
fun LoginScreenContent(
    loginScreenViewModel: LoginScreenViewModel,
    paddingValues: PaddingValues,
    onNavigateToRegistration: () -> Unit,
    errorCallback: (String) -> Unit
){
    val state = loginScreenViewModel.loginScreenState.collectAsState()

    LaunchedEffect(state.value.error) {
        state.value.error?.let {
            errorCallback(it)
        }
        state.value.error = null
    }

    Column(
        modifier = Modifier
            .padding(paddingValues)
            .fillMaxSize()
            .padding(horizontal = 16.dp)
    ) {

        Spacer(modifier = Modifier.height(24.dp))
        Text(
            text = "C возвращением!",
            style = TextStyle(
                fontSize = 32.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope_extrabold))
            ),
            color = Color.Black
        )
        Spacer(modifier = Modifier.height(8.dp))
        Text(
            text = "Введите свой email, чтобы начать делать покупки и получать выгодные предложения уже сегодня!",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Neutral_70
        )
        Spacer(modifier = Modifier.height(32.dp))
        FurnitureTextField(
            value = state.value.request.email,
            onValueChange = { loginScreenViewModel.setEmail(it) },
            textLabel = "Email",
            leadingResourceId = R.drawable.mail_icon
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurniturePasswordTextField(
            value = state.value.request.password,
            onValueChange = { loginScreenViewModel.setPassword(it) }
        )
        Spacer(modifier = Modifier.height(16.dp))
        Text(
            text = "Забыли пароль?",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Primary
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurnitureButton(
            text = "Войти"
        ) {
            loginScreenViewModel.login()
        }
        Spacer(modifier = Modifier.height(16.dp))
        Text(
            modifier = Modifier.clickable {
                onNavigateToRegistration()
            },
            text = "У вас нет учетной записи? Зарегистрируйтесь",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Neutral_70
        )
    }

}