package com.example.furintiture.ui.screen.main

import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.wrapContentHeight
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.pager.HorizontalPager
import androidx.compose.foundation.pager.rememberPagerState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Brush
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.motionEventSpy
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import coil3.compose.AsyncImage
import coil3.request.ImageRequest
import coil3.request.crossfade
import com.example.furintiture.R
import com.example.furintiture.ui.screen.bottomsheet.FurnitureAddressBottomSheet
import com.example.furintiture.ui.common.FurnitureHorizontalList
import com.example.furintiture.ui.screen.Detail
import com.example.furintiture.ui.screen.Furniture
import com.example.furintiture.ui.screen.bottomsheet.FurnitureAddressBottomSheetViewModel
import com.example.furintiture.ui.theme.Neutral_20
import com.example.furintiture.ui.theme.Neutral_40
import com.example.furintiture.ui.theme.Primary
import com.example.furintiture.ui.theme.gradientColors
import kotlinx.coroutines.launch

@Composable
fun MainScreen(
    uuid: String,
    onNavigateToFurniture: (Furniture) ->  Unit,
    onNavigateToDetail: (Detail) -> Unit,
) {
    val snackbarHostState  = remember { SnackbarHostState() }
    val coroutineScope = rememberCoroutineScope()
    val viewModel: MainScreenViewModel = viewModel {
        MainScreenViewModel(uuid)
    }
    Scaffold(
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState)
        },
        contentColor = Neutral_20
    ) { paddingValues ->
        MainScreenContent(uuid, viewModel, paddingValues, onNavigateToFurniture, onNavigateToDetail ){
            coroutineScope.launch {
                snackbarHostState.showSnackbar(message = it)
            }
        }
    }
}

@Composable
fun MainScreenContent(
    userUuid: String,
    mainScreenViewModel: MainScreenViewModel,
    paddingValues: PaddingValues,
    onNavigateToFurniture: (Furniture) ->  Unit,
    onNavigateToDetail: (Detail) -> Unit,
    errorCallback: (String) -> Unit,
) {
    val state = mainScreenViewModel.mainScreenState.collectAsState()
    val pagerState = rememberPagerState(pageCount = {state.value.sales.size})
    val showBottomSheet  = remember { mutableStateOf(false) }
    val viewModel: FurnitureAddressBottomSheetViewModel = viewModel()
    LaunchedEffect(state.value.error) {
        state.value.error?.let {
            errorCallback(it)
        }
        state.value.error = null
    }

    Column(
        modifier = Modifier
            .padding(paddingValues)
            .fillMaxSize()
            .padding(bottom = 100.dp,)
    ) {
        Spacer(modifier = Modifier.height(16.dp))
        state.value.profile?.let {
            val text = if(it.address != null) "Доставим по адресу: ${it.address.address}" else  "Выберите адрес доставки"
                Text(
                    modifier =
                    Modifier.padding(horizontal = 16.dp)
                        .clickable {
                            showBottomSheet.value = true
                        }
                    ,
                    text = text,
                    overflow = TextOverflow.Ellipsis,
                    maxLines = 1,
                    style = TextStyle(
                        fontSize = 16.sp,
                        lineHeight = 20.sp,
                        fontFamily = FontFamily(Font(R.font.manrope_extrabold)),
                        color = com.example.furintiture.ui.theme.Text
                    )
                )
            Spacer(modifier = Modifier.height(16.dp))
        }
        if(showBottomSheet.value){
            FurnitureAddressBottomSheet(
                state.value.profile?.address, viewModel
            ) { requestAddress ->
                mainScreenViewModel.addAddress(requestAddress)
                showBottomSheet.value = false
            }
        }
        HorizontalPager(state = pagerState) { page ->
            Box(
                modifier = Modifier
                    .fillMaxWidth()
                    .height(200.dp)
                    .padding(top = 20.dp)
                ,
            ) {
                AsyncImage(
                    modifier = Modifier
                        .fillMaxSize()
                    ,
                    model = ImageRequest
                        .Builder(LocalContext.current)
                        .data(state.value.sales[page].url)
                        .crossfade(true)
                        .build(),
                    placeholder = painterResource(R.drawable.images),
                    contentScale = ContentScale.FillWidth,
                    contentDescription = null
                )
                Box(
                    modifier = Modifier
                        .fillMaxSize()
                        .background(Brush.horizontalGradient(colorStops = gradientColors)))
                Column(
                    verticalArrangement = Arrangement.Center,
                    horizontalAlignment = Alignment.Start,
                    modifier = Modifier
                        .padding(10.dp)
                ) {
                    Text(
                        text = state.value.sales[page].name,
                        style = TextStyle(
                            fontSize = 16.sp,
                            lineHeight = 20.sp,
                            fontFamily = FontFamily(Font(R.font.manrope_extrabold))
                        ),
                        color = Neutral_20
                    )
                    Button(
                        onClick = {},
                        colors = ButtonDefaults.buttonColors().copy(
                            contentColor = Color.White,
                            disabledContentColor = Color.White,
                            containerColor = Color.White,
                            disabledContainerColor = Color.White
                        )
                    ) {
                        Text(
                            "Купить сейчас",
                            style = TextStyle(
                                fontSize = 16.sp,
                                lineHeight = 20.sp,
                                fontFamily = FontFamily(Font(R.font.manrope)),
                                fontWeight = FontWeight.Bold
                            ),
                            color = Primary
                        )
                    }
                }

            }

        }
        Spacer(modifier = Modifier.height(16.dp))
        Row(
            modifier = Modifier
                .wrapContentHeight()
                .fillMaxWidth()
                .padding(8.dp),
            horizontalArrangement = Arrangement.Center
        ) {
            repeat(pagerState.pageCount) { iteration ->
                val color = if (pagerState.currentPage == iteration) Primary else Neutral_40
                Box(
                    modifier = Modifier
                        .padding(2.dp)
                        .clip(CircleShape)
                        .background(color)
                        .size(12.dp)
                )
            }
        }
        Spacer(modifier = Modifier.height(16.dp))
        LazyColumn(
            modifier = Modifier.fillMaxWidth(),
            contentPadding = PaddingValues(16.dp)
        ) {
            items(state.value.shopCategories.size){ item ->
                FurnitureHorizontalList(
                    categoryName = state.value.shopCategories[item].name,
                    categoryFurnitureList = state.value.shopCategories[item].furnitureList,
                    onAllFurnitureClick = { onNavigateToFurniture(Furniture(uuid = userUuid)) }
                ){
                    onNavigateToDetail(Detail(furnitureId = it.id, uuid = userUuid))
                }
            }
        }
    }
}