package com.example.furintiture.ui.screen.registration

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.example.furintiture.R
import com.example.furintiture.data.DataStoreSettings
import com.example.furintiture.ui.common.FurnitureButton
import com.example.furintiture.ui.common.FurniturePasswordTextField
import com.example.furintiture.ui.common.FurnitureTextField
import com.example.furintiture.ui.screen.login.LoginScreenContent
import com.example.furintiture.ui.screen.login.LoginScreenViewModel
import com.example.furintiture.ui.theme.Neutral_20
import com.example.furintiture.ui.theme.Neutral_70
import com.example.furintiture.ui.theme.Primary
import kotlinx.coroutines.launch

@Composable
fun RegistrationScreen(
    dataStoreSettings: DataStoreSettings,
    onNavigateToLogin: () -> Unit
) {
    val snackbarHostState  = remember { SnackbarHostState() }
    val coroutineScope = rememberCoroutineScope()
    val viewModel: RegistrationScreenViewModel = viewModel { RegistrationScreenViewModel(dataStoreSettings) }
    Scaffold(
        snackbarHost = {
            SnackbarHost(hostState = snackbarHostState)
        },
        contentColor = Neutral_20
    ) { paddingValues ->
        RegisterScreenContent(viewModel, paddingValues, onNavigateToLogin){
            coroutineScope.launch {
                snackbarHostState.showSnackbar(message = it)
            }
        }
    }
}

@Composable
fun RegisterScreenContent(
    registrationScreenViewModel: RegistrationScreenViewModel,
    paddingValues: PaddingValues,
    onNavigateToLogin: () -> Unit,
    errorCallback: (String) -> Unit
){
    val state = registrationScreenViewModel.registrationScreenState.collectAsState()

    LaunchedEffect(state.value.error) {
        state.value.error?.let {
            errorCallback(it)
        }
        state.value.error = null
    }

    Column(
        modifier = Modifier
            .padding(paddingValues)
            .fillMaxSize()
            .padding(horizontal = 16.dp)
    ) {

        Spacer(modifier = Modifier.height(24.dp))
        Text(
            text = "Создать аккаунт",
            style = TextStyle(
                fontSize = 32.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope_extrabold))
            ),
            color = Color.Black
        )
        Spacer(modifier = Modifier.height(8.dp))
        Text(
            text = "Введите свои данные ниже, чтобы начать совершать покупки.",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Neutral_70
        )
        Spacer(modifier = Modifier.height(32.dp))
        FurnitureTextField(
            value = state.value.request.firstName,
            onValueChange = { registrationScreenViewModel.setFirstName(it) },
            textLabel = "Имя",
            leadingResourceId = R.drawable.user_icon
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurnitureTextField(
            value = state.value.request.lastName,
            onValueChange = { registrationScreenViewModel.setLastName(it) },
            textLabel = "Фамиля",
            leadingResourceId = R.drawable.user_icon
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurnitureTextField(
            value = state.value.request.email,
            onValueChange = { registrationScreenViewModel.setEmail(it) },
            textLabel = "Email",
            leadingResourceId = R.drawable.mail_icon
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurniturePasswordTextField(
            value = state.value.request.password,
            onValueChange = { registrationScreenViewModel.setPassword(it) }
        )
        Spacer(modifier = Modifier.height(16.dp))
        Text(
            text = "Нажимая кнопку Зарегистрироваться, вы подтверждаете, что ознакомились и согласились с нашими Условиями использования и Политикой конфиденциальности.",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Primary
        )
        Spacer(modifier = Modifier.height(16.dp))
        FurnitureButton(
            text = "Зарегистрироваться"
        ) {
            registrationScreenViewModel.register()
        }
        Spacer(modifier = Modifier.height(16.dp))
        Text(
            modifier = Modifier.clickable {
                onNavigateToLogin()
            },
            text = "Войти",
            style = TextStyle(
                fontSize = 16.sp,
                lineHeight = 20.sp,
                fontFamily = FontFamily(Font(R.font.manrope))
            ),
            color = Neutral_70
        )
    }
}